@@ -9,13 +9,9 @@ from os.path import basename
 from os.path import dirname
 from os.path import isfile
 from os.path import join
-from os.path import split
 from re import sub as re_sub
 from typing import Callable
-from typing import Dict
-from typing import List
 from typing import Optional
-from typing import Tuple
 from typing import Union
 from zipfile import ZipFile
 
@@ -102,9 +98,9 @@ def load_user(username: str, _cache=None) -> Optional[dict]:
 
 
 @cache
-def load_user_stats(username: str, _cache=None) -> Dict[str, int]:
+def load_user_stats(username: str, _cache=None) -> dict[str, int]:
     username = clean_username(username)
-    stats: Dict[str, int] = {}
+    stats: dict[str, int] = {}
     with FADatabase(db_path) as db:
         stats["gallery"] = db.submissions.select(
             {"replace(lower(author), '_', '')": username, "folder": "gallery"}, ["count(ID)"]
@@ -125,50 +121,54 @@ def load_user_stats(username: str, _cache=None) -> Dict[str, int]:
 
 
 @cache
-def load_item(table: str, id_: int) -> Tuple[Dict[str, Union[str, int, List[str]]], int, int]:
+def load_item(table: str, id_: int) -> Optional[dict[str, Union[str, int, list[str]]]]:
     global db_path
-
-    item: Optional[dict]
-    prev_id: int
-    next_id: int
     with FADatabase(db_path) as db:
         db_table: FADatabaseTable = db[table]
-        item = db_table[id_]
-        prev_id, next_id = db_table.select(
+        return db_table[id_]
+
+
+@cache
+def load_prev_next(table: str, id_: int) -> tuple[int, int]:
+    global db_path
+    with FADatabase(db_path) as db:
+        item: Optional[dict] = load_item(table, id_)
+        return db[table].select(
             {"AUTHOR": item["AUTHOR"]},
             ["LAG(ID, 1, 0) over (order by ID)", "LEAD(ID, 1, 0) over (order by ID)"],
             order=[f"ABS(ID - {id_})"],
             limit=1
         ).fetchone() if item else (0, 0)
 
-    return item, prev_id, next_id
+
+@cache
+def load_files_folder() -> str:
+    with FADatabase(db_path) as db:
+        return join(dirname(db_path), db.settings["FILESFOLDER"])
 
 
 @cache
-def load_submission_file(id_: int) -> Tuple[Optional[str], str]:
-    global db_path
+def load_submission_file(id_: int) -> tuple[Optional[str], int, str, str]:
+    if (sub := load_item(submissions_table, id_)) is None:
+        return None, 0, "", ""
 
-    sub, _, _ = load_item(submissions_table, id_)
+    sub_file, sub_thumb = "", ""
+    if sub["FILESAVED"] != 0:
+        sub_folder: str = join(load_files_folder(), tiered_path(id_))
+        sub_file = join(sub_folder, "submission" + f".{(e := sub['FILEEXT'])}" * bool(e))
+        sub_thumb = join(sub_folder, "thumbnail.jpg")
 
-    if sub is None:
-        return None, ""
-
-    with FADatabase(db_path) as db:
-        sub_file: str = join(
-            dirname(db_path), db.settings["FILESFOLDER"],
-            *split(tiered_path(id_)), "submission" + f".{(e := sub['FILEEXT'])}" * bool(e))
-
-    return sub["TYPE"], sub_file
+    return sub["TYPE"], sub["FILESAVED"], sub_file, sub_thumb
 
 
 @cache
 def search_table(table: str, sort: str, order: str, params_serialised: str = "{}", force: bool = False, _cache=None):
     global db_path
 
-    cols_results: List[str] = []
-    cols_list: List[str] = []
+    cols_results: list[str] = []
+    cols_list: list[str] = []
     col_id: str = ""
-    params: Dict[str, List[str]] = json_loads(params_serialised)
+    params: dict[str, list[str]] = json_loads(params_serialised)
 
     if table in ("submissions", "journals"):
         cols_results = ["ID", "AUTHOR", "DATE", "TITLE"]
@@ -184,7 +184,7 @@ def search_table(table: str, sort: str, order: str, params_serialised: str = "{}
 
     with FADatabase(db_path) as db:
         db_table: FADatabaseTable = db[table]
-        cols_table: List[str] = db_table.columns
+        cols_table: list[str] = db_table.columns
 
         if not params and not force:
             return [], cols_table, cols_results, cols_list, col_id, sort, order
@@ -243,7 +243,7 @@ def user(username: str):
         return redirect(f"/user/{username_clean}")
 
     user_entry: Optional[dict] = load_user(username, _cache=m_time(db_path))
-    user_stats: Dict[str, int] = load_user_stats(username, _cache=m_time(db_path))
+    user_stats: dict[str, int] = load_user_stats(username, _cache=m_time(db_path))
 
     return render_template(
         "user.html",
@@ -329,7 +329,7 @@ def search(table: str):
     if table not in ("submissions", "journals", "users"):
         return error(f"Table {table} not found.", 404)
 
-    params: Dict[str, List[str]] = {
+    params: dict[str, list[str]] = {
         k: request.args.getlist(k) for k in sorted(map(str.lower, request.args.keys()))
         if k not in ("page", "limit", "sort", "order", "view")
     }
@@ -337,9 +337,9 @@ def search(table: str):
     if params and request.path.startswith("/browse/"):
         return redirect(url_for("search", table=table, **{k: request.args.getlist(k) for k in request.args}))
 
-    results: List[dict]
-    columns_results: List[str]
-    columns_list: List[str]
+    results: list[dict]
+    columns_results: list[str]
+    columns_list: list[str]
     column_id: str
     limit: int = int(request.args.get("limit", 48))
     page: int = int(request.args.get("page", 1))
@@ -428,9 +428,7 @@ def submission_view(id_: int):
 @cache
 @app.route("/submission/<int:id_>/")
 def submission(id_: int):
-    sub, prev_id, next_id = load_item(submissions_table, id_)
-
-    if sub is None:
+    if (sub := load_item(submissions_table, id_)) is None:
         return error(
             f"Submission not found.<br>{button(f'https://www.furaffinity.net/view/{id_}', 'Open on Fur Affinity')}",
             404
@@ -440,8 +438,8 @@ def submission(id_: int):
         "submission.html",
         title=f"{app.name} · {sub['TITLE']} by {sub['AUTHOR']}",
         submission=sub,
-        prev=prev_id,
-        next=next_id
+        prev=(prev_next := load_prev_next(submissions_table, id_))[0],
+        next=prev_next[1]
     )
 
 
@@ -449,7 +447,7 @@ def submission(id_: int):
 @app.route("/submission/<int:id_>/file/")
 @app.route("/submission/<int:id_>/file/<filename>")
 def submission_file(id_: int, filename: str = None):
-    if not (sub_file := load_submission_file(id_)[1]):
+    if not (sub_file := load_submission_file(id_)[2]):
         return abort(404)
     elif isfile(sub_file):
         return send_file(sub_file, attachment_filename=filename)
@@ -465,19 +463,19 @@ def submission_file(id_: int, filename: str = None):
 @app.route("/submission/<int:id_>/thumbnail/<int:x>x<int:y>/")
 @app.route("/submission/<int:id_>/thumbnail/<int:x>x<int:y>/<string:filename>")
 def submission_thumbnail(id_: int, x: int = 150, y: int = None, filename: str = None):
-    sub_type, sub_file = load_submission_file(id_)
+    sub_type, sub_filesaved, sub_file, sub_thumb = load_submission_file(id_)
     y = x if y is None else y
 
-    if sub_type is None:
+    if sub_type is None or sub_filesaved == 0:
         return abort(404)
-    elif isfile(path := join(dirname(sub_file), "thumbnail.jpg")):
+    elif sub_filesaved % 10 == 1 and isfile(sub_thumb):
         f_obj: BytesIO = BytesIO()
-        with Image.open(path) as img:
+        with Image.open(sub_thumb) as img:
             img.thumbnail((x, y)) if img.width > x or img.height > y else None
             img.save(f_obj, "jpeg")
         f_obj.seek(0)
         return send_file(f_obj, attachment_filename=filename, mimetype="image/jpeg")
-    elif sub_type == "image" and isfile(sub_file):
+    elif sub_filesaved >= 10 and sub_type == "image" and isfile(sub_file):
         f_obj: BytesIO = BytesIO()
         with Image.open(sub_file) as img:
             img.thumbnail((x, y))
@@ -492,16 +490,14 @@ def submission_thumbnail(id_: int, x: int = 150, y: int = None, filename: str =
 @app.route("/submission/<int:id_>/zip/")
 @app.route("/submission/<int:id_>/zip/<filename>")
 def submission_zip(id_: int, filename: str = None):
-    sub, _, _ = load_item(submissions_table, id_)
-
-    if sub is None:
+    if (sub := load_item(submissions_table, id_, False)) is None:
         return abort(404)
 
-    _, sub_file = load_submission_file(id_)
+    _, sub_filesaved, sub_file, _ = load_submission_file(id_)
     f_obj: BytesIO = BytesIO()
 
     with ZipFile(f_obj, "w") as z:
-        if isfile(sub_file):
+        if sub_filesaved >= 10 and isfile(sub_file):
             z.writestr(basename(sub_file), open(sub_file, "rb").read())
         z.writestr("description.html", sub["DESCRIPTION"].encode())
         z.writestr("metadata.json", json_dumps({k: v for k, v in sub.items() if k != "DESCRIPTION"}).encode())
